package	jp.co.sds_corp.hibiya.adapter;

import java.util.LinkedHashMap;
import java.util.Map;
import java.util.Vector;

import jp.co.sds_corp.hibiya.connector.ConnectionEvent;
import jp.co.sds_corp.hibiya.connector.IConnectionListener;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.hibiya.server.IChecker;
import jp.co.sds_corp.hibiya.server.ICheckerListener;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.daemon.AbstractService;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
* {@link IChecker}の情報を取得するアダプタの基底クラスです。
*/
public abstract class AbstractAdapter extends AbstractService
	implements IAdapter
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	LinkedHashMap<String,IChecker> mapChk = new LinkedHashMap<String,IChecker>();
	Vector<ICheckerListener> vecLsn = new Vector<ICheckerListener>();
	Vector<IConnectionListener> vecCon = new Vector<IConnectionListener>();
	boolean	started = false;	// 開始フラグ

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
	}

	/**
	* 開始フラグを設定します。
	* @param	var		設定するデータ
	*/
	public void setStarted( boolean var ) {
		this.started = var;
	}
	/**
	* 開始フラグを取得します。
	* @return	開始フラグ
	*/
	public boolean isStarted() {
		return started;
	}

	/**
	* AdapterにICheckerを追加します。
	* @param	chk		追加するモニタ
	*/
	public void addChecker( IChecker chk ) {
		mapChk.put( chk.getName(), chk );
		chk.addCheckerListener( this );
	}

	/**
	* AdapterからICheckerを削除します。
	* @param	chk		削除するモニタ
	*/
	public void removeChecker( IChecker chk ) {
		chk.removeCheckerListener( this );
		mapChk.remove( chk.getName() );
	}

	/**
	* 登録されているモニタの一覧を返します。
	*/
	public Map<String,IChecker> getCheckerMap() {
		return mapChk;
	}

	public void checkerChanged( CheckerChangeEvent ev ) throws Exception {
		fireCheckerChange( ev );
	}

	/**
	* 状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addCheckerListener( ICheckerListener l ) {
		vecLsn.add( l );
	}

	/**
	* 状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeCheckerListener( ICheckerListener l ) {
		vecLsn.remove( l );
	}

	/**
	* 状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireCheckerChange( CheckerChangeEvent ev ) throws Exception {
		Vector<ICheckerListener> vecErr = new Vector<ICheckerListener>();
		ICheckerListener l;
		for ( int i = 0; i < vecLsn.size(); i++ ) {
			l = (ICheckerListener )vecLsn.get( i );
			try {
				l.checkerChanged( ev );
			} catch ( Exception ex ) {
				logger.error( this, "checker notify error", ex );
				vecErr.add( l );
			}
		}
		for ( int i = 0; i < vecErr.size(); i++ ) {
			l = (ICheckerListener )vecErr.get( i );
			vecLsn.remove( l );
		}
	}

	/**
	* 接続状態の変更を受け取るオブジェクトを追加します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void addConnectionListener( IConnectionListener l ) {
		vecCon.add( l );
	}

	/**
	* 接続状態の変更を受け取るオブジェクトを削除します。
	* @param	l		状態の変更を受け取るオブジェクト
	*/
	public void removeConnectionListener( IConnectionListener l ) {
		vecCon.remove( l );
	}

	/**
	* 接続状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireConnected( ConnectionEvent ev ) {
		IConnectionListener l;
		for ( int i = 0; i < vecCon.size(); i++ ) {
			l = (IConnectionListener )vecCon.get( i );
			try {
				l.connected( ev );
			} catch ( Exception ex ) {
				logger.error( this, "connect notify error", ex );
			}
		}
	}

	/**
	* 接続状態変更イベントを通知します。
	* @param	ev		状態変更イベント
	*/
	public void fireDisconnected( ConnectionEvent ev ) {
		IConnectionListener l;
		for ( int i = 0; i < vecCon.size(); i++ ) {
			l = (IConnectionListener )vecCon.get( i );
			try {
				l.disconnected( ev );
			} catch ( Exception ex ) {
				logger.error( this, "disconnect notify error", ex );
			}
		}
	}

}
