package	jp.co.sds_corp.hibiya.action;

import java.io.BufferedWriter;
import java.io.FileWriter;
import java.io.PrintWriter;

import jp.co.sds_corp.hibiya.layout.EventLayout;
import jp.co.sds_corp.hibiya.layout.ILayout;
import jp.co.sds_corp.hibiya.server.CheckerChangeEvent;
import jp.co.sds_corp.util.config.IConfiguration;
import jp.co.sds_corp.util.log.CmnLogFactory;
import jp.co.sds_corp.util.log.CmnLogger;

/**
受信したイベントを、ファイルに保存するアクションです。
XMLは以下の形式です。
<TABLE BORDER="1">
	<TR><TH>要素</TH><TH>必須</TH><TH>説明</TH><TH>備考</TH></TR>
	<TR><TD>File</TD><TD>※</TD><TD>出力先</TD><TD>&nbsp;</TD></TR>
	<TR>
		<TD>Layout</TD><TD>&nbsp;</TD>
		<TD>formatに使用するLayout</TD>
		<TD>指定されない場合はメッセージのみ出力</TD>
	</TR>
</TABLE>
<PRE>
&lt;Action class="jp.co.sds_corp.hibiya.action.FileAction"&gt;
	&lt;File&gt;log/hibiya.dat&lt;/File&gt;
	&lt;Layout class="jp.co.sds_corp.hibiya.layout.EventLayout"&gt;
		%d %h %f:%p [%m]
	&lt;/Layout&gt;
&lt;/Action&gt;
</PRE>
*/
public class FileAction extends AbstractAction
{
	static final CmnLogger logger = CmnLogFactory.getLogger();
	String	file;
	ILayout	fnLayout = new EventLayout();

	/**
	* 設定情報で初期化します。
	* @param	config	設定情報
	*/
	public void init( IConfiguration config ) throws Exception {
		super.init( config );
		setFile( config.get( "File" ));
		fnLayout.setFormat( getFile() );
	}

	/**
	* アクションを実行します。
	*/
	public void action( CheckerChangeEvent ev ) {
		try {
			String s;
			if ( getLayout() == null ) {
				s = ev.getMessage();
			} else {
				s = getLayout().format( ev );
			}

			String file = fnLayout.format( ev );
			PrintWriter out = new PrintWriter(
				new BufferedWriter( new FileWriter( file, true )));
			out.print( s );
			out.flush();
			out.close();
		} catch ( Exception e ) {
			logger.error( this, "error", e );
		}
	}

	/**
	* 設定内容の文字列表現を返します。
	*/
	public String paramString() {
		String ret = super.paramString();
		ret += file;
		return ret;
	}

	/**
	* 設定されている内容をXML形式で出力します。
	* @param	config	親ノード
	*/
	public void saveConfig( IConfiguration config ) {
		super.saveConfig( config );
		IConfiguration c;
		c = config.addChild( "File", getFile() );
	}

	/**
	* ファイル名(file)を設定します。
	* @param	var		設定するデータ
	*/
	public void setFile( String var ) {
		this.file = var ;
		fnLayout.setFormat( var );
	}
	/**
	* ファイル名(file)を取得します。
	* @return	fileの内容
	*/
	public String getFile() {
		return file;
	}

}
